#!/usr/bin/env python
# -*- coding: utf-8 -*-

import warnings
import os
import os.path
import pkg_resources
from collections import namedtuple
from distutils import sysconfig

__all__ = ['crypto_sign', 'crypto_sign_open', 'crypto_sign_keypair', 'Keypair',
           'PUBLICKEYBYTES', 'SECRETKEYBYTES', 'SIGNATUREBYTES']

from . import _ed25519
ffi = _ed25519.ffi

PUBLICKEYBYTES=32
SECRETKEYBYTES=64
SIGNATUREBYTES=64

try: # Convert bytes (str) to a list of integers # pragma nocover 
    unicode
    def numlist(b):
        return map(ord, b)        
    def _ffi_tobytes(c, size):
        return ffi.buffer(c)[:size]
except NameError: # pragma nocover
    def numlist(b):
        return list(b)
    def _ffi_tobytes(c, size):
        return ffi.buffer(c).tobytes()[:size]
    
Keypair = namedtuple('Keypair', ('vk', 'sk')) # verifying key, secret key

def crypto_sign_keypair(seed=None):
    """Return (verifying, secret) key from a given seed, or os.urandom(32)"""
    vk = ffi.new('unsigned char[32]')
    sk = ffi.new('unsigned char[64]')
    if seed is None:
        seed = os.urandom(PUBLICKEYBYTES)
    else:
        warnings.warn("ed25519ll should choose random seed.",
                      RuntimeWarning)
    if len(seed) != 32:
        raise ValueError("seed must be 32 random bytes or None.")
    s = ffi.new('unsigned char[32]', numlist(seed))
    rc = _ed25519.crypto_sign_keypair(vk, sk, s)
    if rc != 0: # pragma no cover (no other return statement in C)
        raise ValueError("rc != 0", rc)
    return Keypair(_ffi_tobytes(vk, 32), _ffi_tobytes(sk, 64))


def crypto_sign(msg, sk):
    """Return signature+message given message and secret key.
    The signature is the first SIGNATUREBYTES bytes of the return value.
    A copy of msg is in the remainder."""
    assert len(sk) == SECRETKEYBYTES
    sk = ffi.new('unsigned char[]', numlist(sk))
    m = ffi.new('unsigned char[]', numlist(msg))
    sig_and_msg = ffi.new('unsigned char[]', (len(msg) + SIGNATUREBYTES))
    sig_and_msg_len = ffi.new('unsigned long long[1]')
    rc = _ed25519.crypto_sign(sig_and_msg, sig_and_msg_len, m, len(m), sk)
    if rc != 0: # pragma no cover (no other return statement in C)
        raise ValueError("rc != 0", rc)
    return _ffi_tobytes(sig_and_msg, sig_and_msg_len[0])


def crypto_sign_open(signed, vk):
    """Return message given signature+message and the verifying key."""
    assert len(vk) == PUBLICKEYBYTES
    sm = ffi.new('unsigned char[]', numlist(signed))
    vk = ffi.new('unsigned char[]', numlist(vk))
    newmsg = ffi.new('unsigned char[]', len(signed))
    newmsg_len = ffi.new('unsigned long long[1]')
    rc = _ed25519.crypto_sign_open(newmsg, newmsg_len, sm, len(sm), vk)
    if rc != 0:
        raise ValueError("rc != 0", rc)    
    return _ffi_tobytes(newmsg, newmsg_len[0])




#!/usr/bin/env python

import glob
import os
import os.path
import platformer
import distutils.util

plat_name = distutils.util.get_platform().replace('-', '_')
output = os.path.abspath(os.path.join('ed25519ll', '_ed25519_%s' % plat_name))
os.chdir("ed25519-supercop-ref10")
platformer.platform.compile(glob.glob("*.c"), 
        platformer.ExternalCompilationInfo(compile_extra=['-march=native']), 
        outputfilename=output, standalone=False)



# -*- coding: utf-8 -*-

from nose.tools import raises
import ed25519ll 

def test_ed25519ll():
    msg = b"The rain in Spain stays mainly on the plain"
    kp = ed25519ll.crypto_sign_keypair()
    signed = ed25519ll.crypto_sign(msg, kp.sk)    
    
    @raises(ValueError)
    def open_corrupt(): 
        corrupt = signed[ed25519ll.SIGNATUREBYTES:] + b'U' + signed[ed25519ll.SIGNATUREBYTES+1:]
        ed25519ll.crypto_sign_open(corrupt, kp.vk)                
    open_corrupt()
    
    @raises(AssertionError)
    def short_sk():
        signed = ed25519ll.crypto_sign(msg, kp.sk[:-1])
    short_sk()
    
    ed25519ll.crypto_sign_open(signed, kp.vk)
    
def test_cover_warn_seed():
    ed25519ll.crypto_sign_keypair(b'*'*32)
    
@raises(ValueError)
def test_bad_seed_size():
    ed25519ll.crypto_sign_keypair(b'*'*31)

if __name__ == "__main__":
    test_ed25519ll()



import pkg_resources
import sysconfig
import os.path

from distutils.util import get_platform
from cffi import FFI
ffi = FFI()

decl = """
    extern int crypto_sign(unsigned char *, unsigned long long *,
        const unsigned char *, unsigned long long, const unsigned char *);
    
    extern int crypto_sign_open(unsigned char *, unsigned long long *, 
        const unsigned char *, unsigned long long, const unsigned char *);
    
    extern int crypto_sign_keypair(unsigned char *pk, unsigned char *sk, 
        unsigned char *seed);
"""

ffi.cdef(decl)

verify = False

plat_name = get_platform().replace('-', '_')
so_suffix = sysconfig.get_config_var('SO')
library_dir = os.path.abspath(pkg_resources.resource_filename('ed25519ll', ''))
if not verify: # pragma no cover
    lib_filename = pkg_resources.resource_filename('ed25519ll', 
                                                   '_ed25519_%s%s' %
                                                   (plat_name, so_suffix))
    _ed25519 = ffi.dlopen(os.path.abspath(lib_filename))
else: # pragma nocover
    so_prefix = ""
    if os.name == "posix":
        so_prefix = ":" # to link with libraries without 'lib' prefix
    _ed25519 = ffi.verify(decl,
                          libraries=["%s_ed25519_%s%s" % (so_prefix,
                                                          plat_name,
                                                          so_suffix)],
                          library_dirs=[library_dir],
                          export_symbols=["crypto_sign",
                                          "crypto_sign_open",
                                          "crypto_sign_keypair"],
                          )

crypto_sign_keypair = _ed25519.crypto_sign_keypair
crypto_sign = _ed25519.crypto_sign
crypto_sign_open = _ed25519.crypto_sign_open



import os
import sys
import glob

from setuptools import setup, find_packages
from distutils.core import Extension
from distutils.util import get_platform

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()

plat_name = get_platform().replace('-', '_')

setup(name='ed25519ll',
      version='0.4',
      description='A low-level cffi wrapper for Ed25519 digital signatures.',
      long_description=README + '\n\n' + CHANGES,
      classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Topic :: Security :: Cryptography",
        ],
      author='Daniel Holth',
      author_email='dholth@fastmail.fm',
      url='http://bitbucket.org/dholth/ed25519ll/',
      keywords='ed25519',
      license='MIT',
      packages=['ed25519ll'],
      include_package_data=True,
      zip_safe=False,
      setup_requires=['cffi'],
      install_requires=['cffi'],
      tests_require=['nose'],
      test_suite='nose.collector',
      ext_modules=[
               Extension('ed25519ll._ed25519_%s' % plat_name,
                         sources=[
                             'ed25519-supercop-ref10/ge_frombytes.c',
                             'ed25519-supercop-ref10/fe_frombytes.c',
                             'ed25519-supercop-ref10/ge_tobytes.c',
                             'ed25519-supercop-ref10/fe_sq.c',
                             'ed25519-supercop-ref10/ge_scalarmult_base.c',
                             'ed25519-supercop-ref10/sc_reduce.c',
                             'ed25519-supercop-ref10/ge_p2_0.c',
                             'ed25519-supercop-ref10/ge_sub.c',
                             'ed25519-supercop-ref10/fe_sub.c',
                             'ed25519-supercop-ref10/ge_p2_dbl.c',
                             'ed25519-supercop-ref10/ge_double_scalarmult.c',
                             'ed25519-supercop-ref10/fe_0.c',
                             'ed25519-supercop-ref10/ge_p3_to_p2.c',
                             'ed25519-supercop-ref10/ge_precomp_0.c',
                             'ed25519-supercop-ref10/fe_cmov.c',
                             'ed25519-supercop-ref10/sc_muladd.c',
                             'ed25519-supercop-ref10/fe_isnegative.c',
                             'ed25519-supercop-ref10/ge_p3_dbl.c',
                             'ed25519-supercop-ref10/ge_add.c',
                             'ed25519-supercop-ref10/fe_neg.c',
                             'ed25519-supercop-ref10/ge_p3_0.c',
                             'ed25519-supercop-ref10/fe_1.c',
                             'ed25519-supercop-ref10/ge_madd.c',
                             'ed25519-supercop-ref10/fe_tobytes.c',
                             'ed25519-supercop-ref10/sign.c',
                             'ed25519-supercop-ref10/fe_copy.c',
                             'ed25519-supercop-ref10/ge_p1p1_to_p2.c',
                             'ed25519-supercop-ref10/fe_isnonzero.c',
                             'ed25519-supercop-ref10/open.c',
                             'ed25519-supercop-ref10/fe_sq2.c',
                             'ed25519-supercop-ref10/ge_msub.c',
                             'ed25519-supercop-ref10/fe_add.c',
                             'ed25519-supercop-ref10/fe_mul.c',
                             'ed25519-supercop-ref10/fe_pow22523.c',
                             'ed25519-supercop-ref10/fe_invert.c',
                             'ed25519-supercop-ref10/ge_p3_tobytes.c',
                             'ed25519-supercop-ref10/ge_p1p1_to_p3.c',
                             'ed25519-supercop-ref10/ge_p3_to_cached.c',
                             'ed25519-supercop-ref10/sha512-blocks.c',
                             'ed25519-supercop-ref10/sha512-hash.c',
                             'ed25519-supercop-ref10/verify.c',
                             'ed25519-supercop-ref10/keypair.c'],
                         include_dirs=['ed25519-supercop-ref10', ],
                         export_symbols=["crypto_sign",
                                         "crypto_sign_open",
                                         "crypto_sign_keypair"],
                         compile_extra=['-march=native']),
               ],
      )

